package cs;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.text.DateFormat;
import java.text.MessageFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.events.EndElement;
import javax.xml.stream.events.StartElement;
import javax.xml.stream.events.XMLEvent;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.select.Elements;

public class BSCreditExtracter extends DefaultCreditExtracter {

	private DateFormat dateFormat = new SimpleDateFormat("yy-MM-dd");

	private static final String URL_TEMPLATE_USER =
			"http://boincstats.com/en/stats/{0}/user/detail/{1}/lastDays";

	@Override
	public void doExtractCredit(final Project project) {
		System.out.printf("提取%s项目的用户积分...\n", project.name);
		try {
			this.getUserId(project);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
		// extract credit
		this.extract(project);
		// new line
		System.out.println();
	}

	private void loadUserId(final Project project, final Team team) throws Exception {
		String url = StringUtil.resolveURL(project.url, "./team_email_list.php?xml=1&teamid="
				+ team.id);
		String xml = UrlFetchUtil.get(url);
		if (xml == null) {
			throw new RuntimeException("Network error! team_email_list xml is null. team id:"
					+ team.id);
		}
		InputStream is = new ByteArrayInputStream(xml.getBytes("UTF-8"));
		XMLEventReader reader = XMLInputFactory.newInstance()
				.createXMLEventReader(is, "UTF-8");

		StartElement root = XMLUtil.getRootElement(reader);
		if (root == null) {
			throw new RuntimeException("Invalid xml format, root element not found.");
		}
		if (root.getName().getLocalPart().equals("error")) {
			String msg = "team_email_list xml has error. team id:" + team.id;
			StartElement se = null;
			while ((se = XMLUtil.nextTag(reader)) != null) {
				String n = se.getName().getLocalPart();
				if (n.equals("error_string") || n.equals("error_msg")) {
					msg = reader.getElementText();
					break;
				}
			}
			throw new RuntimeException(msg);
		}

		Map<String, User> userNameMap = new HashMap<String, User>(
				project.userList.size() / 75 * 100);
		for (User user : project.userList) {
			if (user.team != team) {
				continue;
			}
			for (String name : user.boincName.split("/")) {
				userNameMap.put(name.toLowerCase(), user);
			}
		}
		for (int i = 0, length = project.userList.size(); reader.hasNext() && i < length;) {
			if (XMLUtil.nextTag(reader, "user") == null) {
				break;
			}
			String name = null, id = null, totalCredit = null;
			while (reader.hasNext()) {
				XMLEvent e = reader.nextEvent();
				if (e.isStartElement()) {
					StartElement se = e.asStartElement();
					String qname = se.getName().getLocalPart();
					if (id == null && qname.equals("id")) {
						id = reader.getElementText();
					} else if (name == null && qname.equals("name")) {
						name = reader.getElementText();
					} else if (totalCredit == null && qname.equals("total_credit")) {
						totalCredit = reader.getElementText();
					}
				} else if (e.isEndElement()) {
					EndElement ee = e.asEndElement();
					if (ee.getName().getLocalPart().equals("user")) {
						break;
					}
				}
				if (name != null && id != null && totalCredit != null) {
					break;
				}
			}
			if (id == null) {
				throw new RuntimeException("Invalid xml format, id not found");
			} else if (name == null) {
				throw new RuntimeException("Invalid xml format, name not found");
			} else if (totalCredit == null) {
				throw new RuntimeException("Invalid xml format, total_credit not found");
			}
			// 检查total_credit，以排除同名称的多个账户
			if (totalCredit.equals("0")) {
				continue;
			}
			User user = userNameMap.get(name.toLowerCase());
			if (user == null || user.id != null) {
				continue;
			}
			i++;
			user.id = id;
		}
		reader.close();
		is.close();
	}

	private void getUserId(final Project project) throws Exception {
		for (Team team : project.teamList) {
			if (team.id == null) {
				team.id = this.getTeamId(team.name, project.url);
			}
			this.loadUserId(project, team);
		}
	}

	private void extract(final Project project) {
		double totalCredit = 0;
		int memberNumber = 0;
		for (User user : project.userList) {
			if (user.id == null) {
				continue;
			}
			String url = MessageFormat.format(URL_TEMPLATE_USER, project.id, user.id);
			String html = UrlFetchUtil.get(url);
			if (html == null) {
				throw new RuntimeException("Network error! 'user lastDays' html is null(" + url
						+ ").");
			}
			Document doc = Jsoup.parse(html);
			Elements trs = doc.select("#tblStats tr");
			// credit
			boolean isFound = false;
			int interval = project.days;
			int i = 1;
			try {
				for (int row = 0; row < 4 && i <= interval; row++) {
					for (int col = 0; col < 10 && i <= interval; col++) {
						if (!isFound) {
							String dateString = trs.get(row * 10).child(1 + col).html()
									.replaceFirst("^(?:\\&nbsp;|\\s*)", "")
									.replaceFirst("(?:\\&nbsp;|\\s*)$", "");
							Date date = this.parseDate(dateString);
							if (date.getTime() == project.endDate.getTime()) {
								String totalCreditString = trs.get(row * 10 + 1).child(1 + col)
										.html()
										.replaceFirst("^(?:\\&nbsp;|\\s*)", "")
										.replaceFirst("(?:\\&nbsp;|\\s*)$", "").replaceAll(",", "");
								if (totalCreditString.equals("")) {
									// 新用户没积分
									i = interval + 1;
									break;
								}
								user.totalCredit = Double.parseDouble(totalCreditString);
								isFound = true;
							}
						}
						if (isFound) {
							String credit = trs.get(row * 10 + 2).child(1 + col).html()
									.replaceFirst("^(?:\\&nbsp;|\\s*)", "")
									.replaceFirst("(?:\\&nbsp;|\\s*)$", "").replaceAll(",", "");
							user.credit += Double.parseDouble(credit);
							i++;
						}
					}
				}
			} catch (RuntimeException e) {
				// 用户ID不存在，通常为新注册用户，而boincstats还没更新
				if (html.contains("User ID doesn't exist")) {
					continue;
				}
				throw e;
			}
			if (i <= interval) {
				throw new RuntimeException("已超出提取时间范围");
			}
			if (user.credit > 0) {
				totalCredit += user.credit;
				memberNumber++;
			}
			System.out.printf("user: %s, credit: %s\n", user.boincName, user.credit);
		}
		project.memberNumber = memberNumber;
		if (!project.challenge.isTeam) {
			project.totalCredit = totalCredit;
		}
	}

	private Date parseDate(final String str) {
		Date d = null;
		try {
			d = dateFormat.parse(str);
		} catch (ParseException e) {
			throw new RuntimeException(e);
		}
		return d;
	}
}
