package cs;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.events.StartElement;
import javax.xml.stream.events.XMLEvent;

public class Console {

	private static final String PROJECT_ID_FILE = "project_id.db";
	private static final String USER_FILE = "user.txt";
	private static final String HISTORY_FILE = "history.txt";
	private static final String CREDIT_FILE = "credit.txt";
	private static final String DEFAULT_TEAM_TC = "Team China";
	private static final String DEFAULT_TEAM_FAH = "China Folding@Home Power";

	private static final PrintStream out = System.out;
	private static final BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
	private static final DateFormat dateFormat = new SimpleDateFormat("yy-MM-dd");

	private Map<String, String> projectIdMap;

	/**
	 * @param args
	 * @throws IOException
	 */
	public static void main(final String[] args) throws Exception {
		// System.setProperty(PROPERTY_RULE_VER14, "true");
		new Console().run();
	}

	public void run() throws Exception {
		this.init();
		// 获取竞赛信息
		Challenge challenge = this.buildChallenge();

		// 提取积分
		CreditExtracter ce = null;
		if (challenge.dataSource == DataSource.BSC) {
			ce = new BSCCreditExtracter();
		} else if (challenge.dataSource == DataSource.BS) {
			ce = new BSCreditExtracter();
		} else if (challenge.dataSource == DataSource.FAH) {
			ce = new FAHCreditExtracter();
		} else {
			ce = new DefaultCreditExtracter();
		}
		for (Project project : challenge.projectList) {
			ce.extractCredit(project);
		}

		// 计算奖分
		new RewardCalculator().evaluate(challenge);

		// 生成输出
		new StatsPrinter().print(challenge);
	}

	private void init() throws IOException {
		projectIdMap = new HashMap<String, String>();
		BufferedReader br = new BufferedReader(new InputStreamReader(
				new FileInputStream(new File(System.getProperty("user.dir"), PROJECT_ID_FILE)),
				"UTF-8"));
		for (String line = null; (line = br.readLine()) != null;) {
			if (line.startsWith("#") || !line.contains("|")) {
				continue;
			}
			String[] arr = line.split("\\|");
			if (arr.length != 2) {
				continue;
			}
			projectIdMap.put(arr[1].trim().toLowerCase(), arr[0].trim());
		}
		br.close();
	}

	private Challenge buildChallenge() throws Exception {
		String input = null;
		List<Map<String, String>> projectsCreditList = null;
		out.println("******输入竞赛信息******");
		Challenge challenge = new Challenge();
		challenge.name = this.answer("竞赛名称");

		// challenge.isTeam
		challenge.isTeam = this.answerBoolean("是否为国际赛");

		// challenge.dataSource
		input = this.answerPattern(
				"积分数据来源(1.boincstats 2.boincstats challenges 3.FAH 4.other)", "1|2|3|4");
		if (input.equalsIgnoreCase("1")) {
			challenge.dataSource = DataSource.BS;
		} else if (input.equalsIgnoreCase("2")) {
			challenge.dataSource = DataSource.BSC;
		} else if (input.equalsIgnoreCase("3")) {
			challenge.dataSource = DataSource.FAH;
		} else {
			challenge.dataSource = DataSource.OTHER;
		}
		if (challenge.dataSource == DataSource.BSC) {
			challenge.dataSource.id = this.answerPattern("boincstats challenges的竞赛ID", "\\d+");
		}

		// challenge.challengeNumber
		input = this.answerPattern("这赛事是全年的第几个竞赛", "[123456789]\\d*");
		challenge.challengeNumber = Integer.valueOf(input);

		// challenge.projectNumber
		input = this.answerPattern("竞赛项目的个数", "[123456789]\\d*");
		challenge.projectNumber = Integer.valueOf(input);
		challenge.projectList = new ArrayList<Project>(challenge.projectNumber);

		out.println();
		out.println("******输入竞赛项目信息******");
		for (int i = 1; i <= challenge.projectNumber; i++) {
			Project project = new Project(challenge);
			challenge.projectList.add(project);

			out.printf("+++项目%d+++\n", i);

			// project.url
			input = this.answer("项目的官方网址");
			if (!input.startsWith("http://") && !input.startsWith("https://")) {
				input = "http://" + input;
			}
			if (!input.endsWith("/")) {
				input += "/";
			}
			out.println("与项目方通信中...");
			this.setProjectInfoFromConfig(project, input);
			if (project.name == null) {
				if (challenge.dataSource == DataSource.BS
						|| challenge.dataSource == DataSource.BSC) {
					throw new RuntimeException("暂时无法获取项目配置信息，请稍候重试");
				}
				// 非boinc项目或项目方暂时无法访问
				project.name = this.answer("项目名称");
			}
			input = projectIdMap.get(project.name.toLowerCase());
			if (input == null
					&& (challenge.dataSource == DataSource.BS
					|| challenge.dataSource == DataSource.BSC)) {
				out.printf("无法找到%s的项目ID, 请更新%s文件.\n", project.name, PROJECT_ID_FILE);
				input = this.answerPattern("你现在需要手动输入项目ID", "\\d+");
			}
			if (input != null) {
				project.id = input;
				out.printf("项目: %s, ID: %s\n", project.name, project.id);
			}
			// project.startDate
			project.startDate = this.answerDate("竞赛开始日期");

			// project.days
			input = this.answerPattern("竞赛持续时间(天)", "[123456789]\\d*");
			project.days = Integer.valueOf(input);

			Calendar calendar = Calendar.getInstance();
			calendar.setTime(project.startDate);
			calendar.add(Calendar.DAY_OF_MONTH, project.days - 1);
			project.endDate = calendar.getTime();

			// 国际赛
			if (challenge.isTeam) {
				// project.teamNumber
				input = this.answerPattern("国际赛中参赛团队数", "[123456789]\\d*");
				project.teamNumber = Integer.valueOf(input);

				// project.challengeRanking
				input = this.answerPattern("国际赛中Team China的排名", "[123456789]\\d*");
				project.challengeRanking = Integer.valueOf(input);

				// project.totalCredit
				input = this.answerPattern("国际赛中Team China的总积分", "[123456789]\\d*\\.?\\d*");
				project.totalCredit = Double.parseDouble(input);
			}

			if (challenge.dataSource == DataSource.OTHER) {
				out.printf("请将%s项目的用户积分记录保存到程序根目录下的%s文件中，按回车键继续...", project.name, CREDIT_FILE);
				in.readLine();
				if (projectsCreditList == null) {
					projectsCreditList = new ArrayList<Map<String, String>>(
							challenge.projectNumber);
				}
				Map<String, String> projectCreditMap = this.loadCredit();
				projectsCreditList.add(projectCreditMap);
				out.printf("读取了%d个用户积分记录.\n", projectCreditMap.size());
			}
		}

		// challenge.days
		Date start = null, end = null;
		for (Project project : challenge.projectList) {
			if (start == null) {
				start = project.startDate;
				end = project.endDate;
				continue;
			}
			if (start.compareTo(project.startDate) > 0) {
				start = project.startDate;
			}
			if (end.compareTo(project.endDate) < 0) {
				end = project.endDate;
			}
		}
		challenge.startDate = start;
		challenge.endDate = end;
		challenge.days = this.intervalDay(start, end);

		Map<String, Integer> historyMap = null;
		out.printf("请将年度所有竞赛用户的竞赛情况记录保存到程序根目录下的%s文件中，按回车键继续...", HISTORY_FILE);
		in.readLine();
		historyMap = this.loadHistory();
		out.printf("读取了%d个记录.\n", historyMap.size());

		out.printf("请将报名用户名单保存到程序根目录下的%s文件中，按回车键继续...", USER_FILE);
		in.readLine();
		int count = this.loadUser(challenge, projectsCreditList, historyMap);
		out.printf("发现%d个报名用户.\n\n", count);
		return challenge;
	}

	static String answer(final String question) throws IOException {
		out.print(question + ": ");
		return in.readLine().trim();
	}

	static boolean answerBoolean(final String question) throws IOException {
		boolean error = true;
		String input = null;
		do {
			input = answer(question + "(y/n)");
			if (!input.equalsIgnoreCase("y") && !input.equalsIgnoreCase("n")) {
				out.println("无效的输入，请重新输入.");
			} else {
				error = false;
			}
		} while (error);
		if (input.equalsIgnoreCase("y")) {
			return true;
		}
		return false;
	}

	static String answerPattern(final String question, final String regex) throws IOException {
		boolean error = true;
		String input = null;
		do {
			input = answer(question);
			if (!input.matches(regex)) {
				out.println("无效的输入，请重新输入.");
			} else {
				error = false;
			}
		} while (error);
		return input;
	}

	static Date answerDate(final String question) throws IOException {
		boolean error = true;
		String input = null;
		Date result = null;
		do {
			input = answer(question + "(e.g. 2012-05-10)");
			try {
				result = parseDate(input);
				error = false;
			} catch (Exception e) {
				out.println("无效的日期，请重新输入.");
			}
		} while (error);
		return result;
	}

	private void setProjectInfoFromConfig(final Project project, final String uri) {
		try {
			String url = "./get_project_config.php";
			url = StringUtil.resolveURL(uri, url);
			String xml = UrlFetchUtil.get(url);
			if (xml == null) {
				throw new RuntimeException("Network error! project_config xml is null.");
			}
			InputStream is = new ByteArrayInputStream(xml.getBytes("UTF-8"));
			XMLEventReader reader = XMLInputFactory.newInstance()
					.createXMLEventReader(is, "UTF-8");

			StartElement root = XMLUtil.getRootElement(reader);
			if (root == null) {
				throw new RuntimeException("Invalid xml format, root element not found.");
			}
			if (root.getName().getLocalPart().equals("error")) {
				String msg = "project_config xml has error.";
				StartElement se = null;
				while ((se = XMLUtil.nextTag(reader)) != null) {
					String n = se.getName().getLocalPart();
					if (n.equals("error_string") || n.equals("error_msg")) {
						msg = reader.getElementText();
						break;
					}
				}
				throw new RuntimeException(msg);
			}

			String rpcUrl = null, name = null;
			while (reader.hasNext()) {
				XMLEvent e = reader.nextEvent();
				if (e.isStartElement()) {
					StartElement se = e.asStartElement();
					String qname = se.getName().getLocalPart();
					if (name == null && qname.equals("name")) {
						name = reader.getElementText().trim();
					} else if (rpcUrl == null && qname.equals("rpc_prefix")) {
						rpcUrl = reader.getElementText();
					}
				}
			}
			reader.close();
			is.close();
			if (name == null) {
				throw new RuntimeException("Invalid xml format, name element not found.");
			}
			if (rpcUrl == null) {
				rpcUrl = uri;
			}
			project.name = name;
			project.url = rpcUrl;
		} catch (Exception e) {
			// NOOP
		}
	}

	private Map<String, String> loadCredit() throws IOException {
		File historyFile = new File(System.getProperty("user.dir"), CREDIT_FILE);
		BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(
				historyFile),
				"UTF-8"));
		Map<String, String> projectCreditMap = new HashMap<String, String>();
		for (String line = null; (line = br.readLine()) != null;) {
			if (line.startsWith("#") || !line.contains("|")) {
				continue;
			}
			String[] arr = line.split("\\|");
			if (arr.length != 3) {
				continue;
			}
			projectCreditMap.put(arr[0].trim().toLowerCase(),
					arr[1].trim() + "|" + arr[2].trim());
		}
		return projectCreditMap;
	}

	private Map<String, Integer> loadHistory() throws IOException {
		File historyFile = new File(System.getProperty("user.dir"), HISTORY_FILE);
		BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(
				historyFile),
				"UTF-8"));
		Map<String, Integer> historyMap = new HashMap<String, Integer>();
		for (String line = null; (line = br.readLine()) != null;) {
			if (line.startsWith("#") || !line.contains("|")) {
				continue;
			}
			String[] arr = line.split("\\|");
			historyMap.put(arr[0].trim().toLowerCase(), Integer.parseInt(arr[1].trim()));
		}
		return historyMap;
	}

	private int loadUser(final Challenge challenge,
			final List<Map<String, String>> projectsCreditList,
			final Map<String, Integer> historyMap) throws IOException {
		File userFile = new File(System.getProperty("user.dir"), USER_FILE);
		BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(userFile),
				"UTF-8"));
		Set<String> nameSet = new HashSet<String>();
		Map<String, Team> teamMap = new HashMap<String, Team>();
		String defaultTeam = DEFAULT_TEAM_TC;
		if (challenge.dataSource == DataSource.FAH) {
			defaultTeam = DEFAULT_TEAM_FAH;
		}
		for (String line = null; (line = br.readLine()) != null;) {
			if (line.startsWith("#") || !line.contains("|")) {
				continue;
			}
			String[] arr = line.split("\\|");
			String forumName = arr[0].trim(), boincName = arr[1].trim();
			String teamName = defaultTeam;
			if (arr.length == 3) {
				teamName = arr[2].trim();
			}
			if (nameSet.contains(forumName)) {
				continue;
			}
			nameSet.add(forumName);

			for (int i = 0; i < challenge.projectList.size(); i++) {
				Project project = challenge.projectList.get(i);
				if (project.userList == null) {
					project.userList = new LinkedList<User>();
				}
				String tid = (project.name + teamName).toLowerCase();
				Team team = teamMap.get(tid);
				if (team == null) {
					team = new Team(teamName);
					teamMap.put(tid, team);
					if (project.teamList == null) {
						project.teamList = new LinkedList<Team>();
					}
					if (!project.teamList.contains(team)) {
						project.teamList.add(team);
					}
				}
				User user = new User(forumName, boincName, team);
				if (challenge.dataSource == DataSource.OTHER) {
					Map<String, String> projectCreditMap = projectsCreditList.get(i);
					String creditString = projectCreditMap.get(boincName.toLowerCase());
					if (creditString == null) {
						break;
					}
					arr = creditString.split("\\|");
					user.totalCredit = Double.parseDouble(arr[0]);
					user.credit = Double.parseDouble(arr[1]);
					if (!challenge.isTeam) {
						project.totalCredit += user.credit;
					}
				}
				/*
				 * 设置用户的参数次数
				 */
				int challengeNumber = 0;
				if (historyMap.containsKey(forumName.toLowerCase())) {
					challengeNumber = historyMap.get(forumName.toLowerCase());
				}
				user.challengeNumber = challengeNumber;

				project.userList.add(user);
			}
		}
		return nameSet.size();
	}

	private static Date parseDate(final String str) {
		Date d = null;
		try {
			d = dateFormat.parse(str);
		} catch (ParseException e) {
			throw new RuntimeException(e);
		}
		return d;
	}

	private int intervalDay(final Date start, final Date end) {
		Calendar c = Calendar.getInstance();
		c.setTime(end);
		c.add(Calendar.DAY_OF_MONTH, 1);
		long time = c.getTime().getTime() - start.getTime();
		return (int) Math.floor(time / 86400000);
	}
}
