package cs;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.events.StartElement;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.select.Elements;

public class DefaultCreditExtracter implements CreditExtracter {

	private static final String URL_TEMPLATE_TEAM =
			"http://boincstats.com/en/stats/{0}/team/detail/{1}";

	public void extractCredit(final Project project) {
		this.doExtractCredit(project);
		System.out.printf("提取%s项目的团队信息...\n", project.name);
		try {
			this.setupTeamInfo(project);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
		// new line
		System.out.println();
	}

	protected void doExtractCredit(final Project project) {
		// NOOP, override only
	}

	private void setupTeamInfo(final Project project) throws Exception {
		Set<Team> teamSet = new HashSet<Team>();
		int count = 0;
		for (User user : project.userList) {
			if (user.credit > 0) {
				count++;
				user.team.memberNumber++;
				teamSet.add(user.team);
			}
		}
		project.memberNumber = count;
		project.teamList = new ArrayList<Team>(teamSet);

		for (Team team : teamSet) {
			if (project.id == null) {// 非boinc项目
				/*
				 * 手动输入团队信息
				 */
				team.ranking = Integer.parseInt(Console.answerPattern(team.name + "团队的国际排名",
						"[123456789]\\d*"));
				team.memberTotalNumber = Integer.parseInt(Console.answerPattern(team.name
						+ "团队的总成员数",
						"[123456789]\\d*"));
				continue;
			}
			if (team.id == null) {
				team.id = this.getTeamId(team.name, project.url);
			}
			this.fetchTeamInfo(project, team);
		}
	}

	private void fetchTeamInfo(final Project project, final Team team) {
		String url = MessageFormat.format(URL_TEMPLATE_TEAM, project.id, team.id);
		String html = UrlFetchUtil.get(url);
		if (html == null) {
			throw new RuntimeException("Network error! 'team detailed stats' html is null(" + url
					+ ").");
		}
		Document doc = Jsoup.parse(html);
		Elements trs = doc.select("#tblStats tr");
		String ranking = trs.get(6).child(1).child(0).html()
				.replaceFirst("^(?:\\&nbsp;|\\s*)", "")
				.replaceFirst("(?:\\&nbsp;|\\s*)$", "").replaceAll(",", "");
		String memberNumber = trs.get(33).child(1).child(0).html()
				.replaceFirst("^(?:\\&nbsp;|\\s*)", "")
				.replaceFirst("(?:\\&nbsp;|\\s*)$", "").replaceAll(",", "");
		team.ranking = Integer.parseInt(ranking);
		team.memberTotalNumber = Integer.parseInt(memberNumber);
	}

	protected String getTeamId(final String name, final String rpcUrl) throws Exception {
		String team_name = null;
		try {
			team_name = URLEncoder.encode(name, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			throw new RuntimeException(e);
		}
		String url = "./team_lookup.php?format=xml&team_name=" + team_name;
		url = StringUtil.resolveURL(rpcUrl, url);
		String xml = UrlFetchUtil.get(url);
		if (xml == null) {
			throw new RuntimeException("Network error! team_lookup xml is null.");
		}
		InputStream is = new ByteArrayInputStream(xml.getBytes("UTF-8"));
		XMLEventReader reader = XMLInputFactory.newInstance()
				.createXMLEventReader(is, "UTF-8");

		StartElement root = XMLUtil.getRootElement(reader);
		if (root == null) {
			throw new RuntimeException("Invalid xml format, root element not found.");
		}
		if (root.getName().getLocalPart().equals("error")) {
			String msg = "team_lookup xml has error.";
			StartElement se = null;
			while ((se = XMLUtil.nextTag(reader)) != null) {
				String n = se.getName().getLocalPart();
				if (n.equals("error_string") || n.equals("error_msg")) {
					msg = reader.getElementText();
					break;
				}
			}
			throw new RuntimeException(msg);
		}

		if (XMLUtil.nextTag(reader, "team") == null) {
			throw new RuntimeException("Invalid xml format, team element not found.");
		}
		if (XMLUtil.nextTag(reader, "id") == null) {
			throw new RuntimeException("Invalid xml format, id element not found.");
		}
		String teamId = reader.getElementText();
		reader.close();
		is.close();
		return teamId;
	}
}
