package cs;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.apache.poi.ss.util.WorkbookUtil;

public class StatsPrinter {

	private static final String TEMPLATE_XLS = "template.xls";
	private static final String OUTPUT_DIR = "output";

	private static final DateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd");

	private Map<String, String> escapeMap;

	public void print(final Challenge challenge) {
		this.initEscapeMap();
		try {
			System.out.println("生成Excel文件...");
			this.printXls(challenge);
			System.out.println("生成Txt文件...");
			this.printTxt(challenge);
			System.out.printf("结果已保存到%s目录下.\n", OUTPUT_DIR);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	private void printTxt(final Challenge challenge) throws IOException {
		Map<Integer, String> rewardMap = new LinkedHashMap<Integer, String>();
		Iterator iterator = null;
		if (challenge.projectNumber > 1) {
			iterator = challenge.userSummaryList.iterator();
		} else {
			iterator = challenge.projectList.get(0).userList.iterator();
		}
		while (iterator.hasNext()) {
			User user = (User) iterator.next();
			if (user.reviseReward == 0) {
				continue;
			}
			String name = rewardMap.get(user.reviseReward);
			if (name == null) {
				name = user.forumName;
			} else {
				name += "," + user.forumName;
			}
			rewardMap.put(user.reviseReward, name);
		}

		File outputFile = new File(System.getProperty("user.dir"), OUTPUT_DIR);
		outputFile.mkdirs();
		outputFile = new File(outputFile, this.createSafeFileName(challenge.name) + ".txt");
		BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(
				outputFile),
				"UTF-8"));
		// 排序
		List<Integer> list = new ArrayList(rewardMap.keySet());
		Collections.sort(list);
		for (Integer reward : list) {
			bw.write(String.valueOf(reward));
			bw.newLine();
			bw.write(rewardMap.get(reward));
			bw.newLine();
			bw.newLine();
		}
		bw.close();
	}

	private void printXls(final Challenge challenge) throws IOException {
		InputStream is = ClassLoader.getSystemResourceAsStream(TEMPLATE_XLS);
		Workbook wb;
		try {
			wb = WorkbookFactory.create(is);
		} catch (InvalidFormatException e) {
			throw new IOException(e);
		}
		for (int i = 0; challenge.projectNumber > 1 && i < challenge.projectNumber; i++) {
			wb.cloneSheet(0);
		}

		CellStyle centerStyle = wb.createCellStyle();
		centerStyle.setAlignment(CellStyle.ALIGN_CENTER);
		centerStyle.setVerticalAlignment(CellStyle.VERTICAL_CENTER);

		int sheetIndex = 0;
		for (Project project : challenge.projectList) {
			Sheet sheet = wb.getSheetAt(sheetIndex++);
			wb.setSheetName(wb.getSheetIndex(sheet), WorkbookUtil.createSafeSheetName(project.name));

			Row row = this.getRow(sheet, 0);
			Cell cell = this.getCell(row, 0);
			String title = challenge.name;
			if (challenge.projectNumber > 1) {
				title += " - " + project.name;
			}
			cell.setCellValue(title);

			int cellIndex = 7;
			for (Team team : project.teamList) {
				cell = this.getCell(row, cellIndex++);
				cell.setCellValue(team.name + "的国际排名");
			}

			row = this.getRow(sheet, 1);
			cell = this.getCell(row, 3);
			String challengeType = "论坛赛";
			if (challenge.isTeam) {
				challengeType = "国际赛";
			}
			cell.setCellValue(challengeType);
			cell.setCellStyle(centerStyle);

			cell = this.getCell(row, 4);
			String dateString = dateFormat.format(project.startDate) + " ~ "
					+ dateFormat.format(project.endDate);
			cell.setCellValue(dateString);
			cell.setCellStyle(centerStyle);

			cell = this.getCell(row, 6);
			cell.setCellValue(challenge.challengeNumber);
			cell.setCellStyle(centerStyle);

			cellIndex = 7;
			for (Team team : project.teamList) {
				cell = this.getCell(row, cellIndex++);
				cell.setCellValue(team.ranking);
				cell.setCellStyle(centerStyle);
			}

			row = this.getRow(sheet, 2);
			cellIndex = 7;
			for (Team team : project.teamList) {
				cell = this.getCell(row, cellIndex++);
				cell.setCellValue(team.name + "的总成员数");
			}

			row = this.getRow(sheet, 3);
			cell = this.getCell(row, 3);
			cell.setCellValue(project.memberNumber);
			cell.setCellStyle(centerStyle);

			cell = this.getCell(row, 4);
			cell.setCellValue(project.totalCredit);
			cell.setCellStyle(centerStyle);

			if (challenge.isTeam) {
				cell = this.getCell(row, 5);
				cell.setCellValue(project.challengeRanking);
				cell.setCellStyle(centerStyle);

				cell = this.getCell(row, 6);
				cell.setCellValue(project.teamNumber);
				cell.setCellStyle(centerStyle);
			}

			cellIndex = 7;
			for (Team team : project.teamList) {
				cell = this.getCell(row, cellIndex++);
				cell.setCellValue(team.memberTotalNumber);
				cell.setCellStyle(centerStyle);
			}

			// 设置列头部
			row = this.getRow(sheet, 4);
			int colIndex = 3;

			if (!challenge.isTeam) {
				cell = this.getCell(row, colIndex++);
				cell.setCellValue("所属团队");
			}

			cell = this.getCell(row, colIndex++);
			cell.setCellValue("个人总积分");

			cell = this.getCell(row, colIndex++);
			cell.setCellValue("个人积分");

			cell = this.getCell(row, colIndex++);
			cell.setCellValue("全年中的参赛次数");

			cell = this.getCell(row, colIndex++);
			cell.setCellValue("竞赛积分");

			if (challenge.projectNumber == 1) {
				cell = this.getCell(row, colIndex++);
				cell.setCellValue("竞赛积分(修正)");
			}

			short rowHeight = row.getHeight();
			int rowIndex = 5;
			for (User user : project.userList) {
				if (user.reward == 0) {
					continue;
				}
				row = this.getRow(sheet, rowIndex++);
				row.setHeight(rowHeight);

				colIndex = 0;
				cell = this.getCell(row, colIndex++);
				cell.setCellValue(user.ranking);

				cell = this.getCell(row, colIndex++);
				cell.setCellValue(user.forumName);

				cell = this.getCell(row, colIndex++);
				cell.setCellValue(user.boincName);

				if (!challenge.isTeam) {
					cell = this.getCell(row, colIndex++);
					cell.setCellValue(user.team.name);
				}

				cell = this.getCell(row, colIndex++);
				cell.setCellValue(user.totalCredit);

				cell = this.getCell(row, colIndex++);
				cell.setCellValue(user.credit);

				cell = this.getCell(row, colIndex++);
				cell.setCellValue(user.challengeNumber);

				cell = this.getCell(row, colIndex++);
				cell.setCellValue(user.reward);

				if (challenge.projectNumber == 1) {
					cell = this.getCell(row, colIndex++);
					cell.setCellValue(user.reviseReward);
				}
			}
		}

		// 汇总表
		if (challenge.projectNumber > 1) {
			Sheet sheet = wb.getSheetAt(sheetIndex);
			wb.setSheetName(wb.getSheetIndex(sheet), WorkbookUtil.createSafeSheetName("summa"));

			Row row = this.getRow(sheet, 0);
			Cell cell = this.getCell(row, 0);
			cell.setCellValue(challenge.name);

			cell = this.getCell(row, 7);
			cell.setCellValue("");

			row = this.getRow(sheet, 1);
			cell = this.getCell(row, 3);
			String challengeType = "论坛赛";
			if (challenge.isTeam) {
				challengeType = "国际赛";
			}
			cell.setCellValue(challengeType);
			cell.setCellStyle(centerStyle);

			cell = this.getCell(row, 4);
			String dateString = dateFormat.format(challenge.startDate) + " ~ "
					+ dateFormat.format(challenge.endDate);
			cell.setCellValue(dateString);
			cell.setCellStyle(centerStyle);

			cell = this.getCell(row, 6);
			cell.setCellValue(challenge.challengeNumber);
			cell.setCellStyle(centerStyle);

			row = this.getRow(sheet, 2);
			cell = this.getCell(row, 7);
			cell.setCellValue("");

			row = this.getRow(sheet, 3);
			cell = this.getCell(row, 3);
			cell.setCellValue(challenge.userSummaryList.size());
			cell.setCellStyle(centerStyle);

			row = this.getRow(sheet, 4);
			int colIndex = 3;
			for (Project project : challenge.projectList) {
				cell = this.getCell(row, colIndex++);
				cell.setCellValue(project.name);
			}

			cell = this.getCell(row, colIndex++);
			cell.setCellValue("竞赛积分");

			cell = this.getCell(row, colIndex);
			cell.setCellValue("竞赛积分(修正)");

			short rowHeight = row.getHeight();
			int rowIndex = 5;
			for (UserSummary user : challenge.userSummaryList) {
				row = this.getRow(sheet, rowIndex++);
				row.setHeight(rowHeight);

				colIndex = 0;
				cell = this.getCell(row, colIndex++);
				cell.setCellValue(user.ranking);

				cell = this.getCell(row, colIndex++);
				cell.setCellValue(user.forumName);

				cell = this.getCell(row, colIndex++);
				cell.setCellValue(user.boincName);

				for (User u : user.userList) {
					cell = this.getCell(row, colIndex++);
					cell.setCellValue(u.reward);
				}

				cell = this.getCell(row, colIndex++);
				cell.setCellValue(user.reward);

				cell = this.getCell(row, colIndex++);
				cell.setCellValue(user.reviseReward);
			}
		}

		File outputFile = new File(System.getProperty("user.dir"), OUTPUT_DIR);
		outputFile.mkdirs();
		outputFile = new File(outputFile, this.createSafeFileName(challenge.name) + ".xls");
		FileOutputStream fos = new FileOutputStream(outputFile);
		wb.write(fos);
		fos.close();
	}

	private void setCenterStyle(final Cell cell) {
		CellStyle style = cell.getCellStyle();

		style.setAlignment(CellStyle.ALIGN_CENTER);
		style.setVerticalAlignment(CellStyle.VERTICAL_CENTER);
	}

	private void initEscapeMap() {
		escapeMap = new HashMap<String, String>();
		escapeMap.put("\\\\", "%5C");
		escapeMap.put("/", "%2F");
		escapeMap.put(":", "%3A");
		escapeMap.put("\\?", "%3F");
		escapeMap.put("\\*", "%2A");
		escapeMap.put("\"", "%22");
		escapeMap.put("<", "%3C");
		escapeMap.put(">", "%3E");
		escapeMap.put("\\|", "%7C");
	}

	private String createSafeFileName(final String fileName) {
		String result = fileName;
		String osName = System.getProperty("os.name");
		if (osName.contains("Windows")) {
			for (String key : escapeMap.keySet()) {
				result = result.replaceAll(key, escapeMap.get(key));
			}
		} else {
			result = result.replaceAll("/", escapeMap.get("/"));
		}
		return result;
	}

	private Row getRow(final Sheet sheet, final int index) {
		Row row = sheet.getRow(index);
		if (row == null) {
			row = sheet.createRow(index);
		}
		return row;
	}

	private Cell getCell(final Row row, final int index) {
		Cell cell = row.getCell(index);
		if (cell == null) {
			cell = row.createCell(index);
		}
		return cell;
	}
}
