package cs;

import java.io.UnsupportedEncodingException;
import java.lang.reflect.Array;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class StringUtil {

	/*
	 * 将字符串转换为Map
	 * 输入的字符串格式参考mapToString
	 */
	public static Map<String, String> stringToMap(final String input) {
		if (input == null || input.equals("")) {
			return Collections.EMPTY_MAP;
		}
		Map<String, String> result = new HashMap<String, String>();
		Pattern pattern = Pattern.compile("#*:");
		int start = 0;
		Matcher matcher = pattern.matcher(input);
		List<String> list = new LinkedList<String>();
		while (matcher.find()) {
			if (matcher.group().length() % 2 == 0) {
				continue;
			}
			list.add(input.substring(start, matcher.end() - 1));
			start = matcher.end();
		}
		list.add(input.substring(start));
		for (int i = 0, length = list.size(); i < length; i += 2) {
			String key = list.get(i).replaceAll("#([#:])", "$1");
			String value = list.get(i + 1).replaceAll("#([#:])", "$1");
			result.put(key, value);
		}
		return result;
	}

	/*
	 * 将Map转换为字符串
	 * 将Map中key和value用冒号(:)连接，然后再将key和value连接所得到的字符串用冒号(:)相互连接，如：key1:value1:key2:value2:...
	 * 其中key和value中的冒号(:)使用#字符进行转义（如：#:），#字符使用双写自身来转义（如：##）
	 */
	public static String mapToString(final Map<String, String> map) {
		List<String> list = new ArrayList<String>(map.size());
		for (String key : map.keySet()) {
			String value = map.get(key);
			key = key.replaceAll("([#:])", "#$1");
			value = value.replaceAll("([#:])", "#$1");
			list.add(key);
			list.add(value);
		}
		return join(list, ":");
	}

	/*
	 * 使用指定的separator连接字符串
	 */
	public static String join(final CharSequence separator, final String... params) {
		return join(Arrays.asList(params), separator);
	}

	/*
	 * 使用指定的separator连接字符串
	 */
	public static String join(final Iterable<String> strings, final CharSequence separator) {
		StringBuilder builder = new StringBuilder();
		for (String str : strings) {
			builder.append(str).append(separator);
		}
		String result = "";
		int length = builder.length();
		if (length > 0) {
			result = builder.substring(0, length - separator.length());
		}
		return result;
	}

	public static String buildUrl(final String url, final Map<String, Object> params) {
		if (params == null || params.isEmpty()) {
			return url;
		}
		URI uri = null;
		try {
			uri = new URI(url);
		} catch (URISyntaxException e) {
			throw new IllegalArgumentException(e);
		}
		String joinChar = "?";
		if (uri.getQuery() != null && !uri.getQuery().equals("")) {
			joinChar = "&";
		}
		List<String> list = new ArrayList<String>(params.size());
		try {
			for (String key : params.keySet()) {
				Object o = params.get(key);
				key = URLEncoder.encode(key, "UTF-8");
				if (o.getClass().isArray()) {
					for (int i = 0, length = Array.getLength(o); i < length; i++) {
						list.add(join("=", key,
								URLEncoder.encode(Array.get(o, i).toString(), "UTF-8")));
					}
				} else {
					list.add(join("=", key, URLEncoder.encode(o.toString(), "UTF-8")));
				}
			}
		} catch (UnsupportedEncodingException e) {
			throw new RuntimeException(e);
		}
		return join(joinChar, url, join(list, "&"));
	}

	/**
	 * Escape an html string. Escaping data received from the client helps to
	 * prevent cross-site script vulnerabilities.
	 * 
	 * @param html
	 *            the html string to escape
	 * @return the escaped string
	 */
	public static String escapeHtml(final String html) {
		if (html == null) {
			return null;
		}
		return html.replaceAll("&", "&amp;").replaceAll("<", "&lt;").replaceAll(">", "&gt;");
	}

	public static String resolveURL(final String url, final String path) {
		URI uri = null;
		try {
			uri = new URI(url);
		} catch (URISyntaxException e) {
			throw new RuntimeException(e);
		}
		return uri.resolve(path).toString();
	}
}
