package cs;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.InetSocketAddress;
import java.net.MalformedURLException;
import java.net.Proxy;
import java.net.URL;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

public class UrlFetchUtil {
	private static final String HTTP_PROXY = "http_proxy";
	private static final String SOCKS_PROXY = "socks_proxy";
	private static final int DEADLINE = 45;
	private static final String USERAGENT = "Mozilla/5.0 Gecko/20100101 Firefox/11.0";
	private static final String PROPERTIE_WAITTIME = "waittime";
	private static final String DEFAULT_WAITTIME = "2.5";

	private static final Map<String, Long> domainMap = new HashMap<String, Long>();

	public static String get(final String uri) {
		URL url = null;
		try {
			url = new URL(uri);
		} catch (MalformedURLException e) {
			throw new IllegalArgumentException(e);
		}
		return get(url);
	}

	public static String get(final URL url) {
		HttpURLConnection connection = null;
		try {
			connection = getConnection(url);
			connection.setDoOutput(false);
			connection.setRequestMethod("GET");
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
		return handleResponse(connection);
	}

	public static String post(final String uri, final Map<String, Object> params) {
		URL url = null;
		try {
			url = new URL(uri);
		} catch (MalformedURLException e) {
			throw new IllegalArgumentException(e);
		}
		return post(url, params);
	}

	public static String post(final URL url, final Map<String, Object> params) {
		HttpURLConnection connection = null;
		try {
			connection = getConnection(url);
			connection.setDoOutput(true);
			connection.setRequestMethod("POST");

			OutputStreamWriter writer = new OutputStreamWriter(connection.getOutputStream(),
					"UTF-8");
			writer.write(StringUtil.buildUrl("", params).substring(1));
			writer.close();
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
		return handleResponse(connection);
	}

	private static HttpURLConnection getConnection(final URL url) throws IOException {
		wait(url);
		HttpURLConnection connection = (HttpURLConnection) url.openConnection(getProxy());
		connection.setConnectTimeout(DEADLINE * 1000);
		connection.setReadTimeout(DEADLINE * 1000);
		connection.setRequestProperty("User-Agent", USERAGENT);
		return connection;
	}

	/*
	 *  防止频繁访问而被ban IP
	 */
	private static void wait(final URL url) {
		String host = url.getHost();
		Long time = domainMap.get(host);
		long waittime = (long) Float.parseFloat(System.getProperty(PROPERTIE_WAITTIME,
				DEFAULT_WAITTIME)) * 1000;
		long nowtime = new Date().getTime();
		if (time != null && nowtime - time <= waittime) {
			try {
				Thread.sleep(waittime);
			} catch (InterruptedException e) {
				throw new RuntimeException(e);
			}
			nowtime += waittime;
		}
		domainMap.put(host, nowtime);
	}

	private static Proxy getProxy() {
		Proxy proxy = Proxy.NO_PROXY;
		String str = System.getProperty(HTTP_PROXY);
		if (str != null) {
			String[] arr = str.split(":");
			if (arr.length == 2) {
				proxy = new Proxy(Proxy.Type.HTTP, new InetSocketAddress(arr[0],
						Integer.parseInt(arr[1])));
			}
		}
		if (proxy == Proxy.NO_PROXY) {
			str = System.getProperty(SOCKS_PROXY);
			if (str != null) {
				String[] arr = str.split(":");
				if (arr.length == 2) {
					proxy = new Proxy(Proxy.Type.SOCKS, new InetSocketAddress(arr[0],
							Integer.parseInt(arr[1])));
				}
			}
		}
		return proxy;
	}

	private static String handleResponse(final HttpURLConnection connection) {
		StringBuilder content = null;
		try {
			if (connection.getResponseCode() != HttpURLConnection.HTTP_OK) {
				return null;
			}
			String encoding = connection.getContentEncoding();
			if (encoding == null) {
				encoding = "UTF-8";
			}
			BufferedReader br = new BufferedReader(new InputStreamReader(
					connection.getInputStream(),
					encoding));
			content = new StringBuilder();
			String line = null;
			while ((line = br.readLine()) != null) {
				content.append(line);
			}
			br.close();
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
		return content.toString();
	}
}
